function [props,values,indp] = idpropch(dat,property,mode)
%IDPROPCH  Return the string of properties and their admissible values  
%       for the object class of object DAT (fiddata).
%       If property is not given, all properties are returned
%       MODE modifies the list of properties:
%       list: mode - 'set', 'all', '', 'child', 'childset'
%       single property: mode - '', 'noerror'

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1998-2003
%       All rights reserved.
%       $Revision: $
%       Last modified: 22-Nov-2003

nargchk(1,3,nargin);
if ~isa(dat,'fiddata'), error('class is not ''fiddata'''), end
if nargin<2, property=''; end
if nargin<3, mode=''; end
%
%All specific FIDDATA Properties
props = ['|Version|InputFreqPoints|OutputFreqPoints|FreqPoints|',...
    'InputDelay|OutputDelay|Fs|InputVariance|OutputVariance|',...
    'Variance|CovVector|CovarianceMatrix|M|',...
    'InputNonlinError|OutputNonlinError|',...
    'InputNonlinVariance|OutputNonlinVariance|NonlinCovVector|NonlinCovarianceMatrix|NonlinM|',...
    'InterExpNonlinCovariance|Coherence|CohVector|',...
    'AllVariances|',...
    'Delays|AllFreqPoints|FreqIndices|FreqNumber|Type|Speciality|N|',...
    'SisoVariance|',...
    'EvenNonexcFrequencies|OddNonexcFrequencies|EvenOutputNonlinError|OddOutputNonlinError|',...
    'OldTbSisoVariance|Fdat|iddat|'];
%Property list also directly handled in fiddata.m
if strncmpi(property,'InterExpCovariance',9)
  warning(sprintf(['Property ''',property,''' is obsolete: it has been changed to ',...
      '''InterExpNonlinCovariance''']))
  property=[property(1:8),'Nonlin',property(9:end)];
end
if nargout>1 %values also needed
  values = {...
      'scalar';...
      'vector or cell array of vectors'; ...
      'vector or cell array of vectors'; ...
      'vector or cell array of vectors (Input + Output)'; ...
      'array';...
      'array';...
      'scalar';...
      'vector (SISO) or cell array (MIMO)';...
      'vector (SISO) or cell array (MIMO)';...
      'vector or cell column vector';...
      'vector (SISO)';...
      '3-dimensional array, 3rd index: all frequency points';...
      'number of processed experiments for variance analysis';...
      ...
      'vector (SISO) or cell array (MIMO)';...
      'vector (SISO) or cell array (MIMO)';...
      ...
      'vector (SISO) or cell array (MIMO)';...
      'vector (SISO) or cell array (MIMO)';...
      'vector (SISO)';...
      '3-dimensional array, 3rd index: all frequency points';...
      'number of processed experiments for (nonlin) variance analysis';...
      ...
      'cell array (expn x expn) of cell array (chn x chn) of cov vectors';...
      '3-dimensional array, 3rd index: all frequency points';...
      'vector (SISO)';...
      ...
      'All (input and output) variances';...
      ...
      'array';...
      'vector or cell array of vectors';...
      'cell array';...
      'read-only scalar';...
      'string';...
      'string';...
      'scalar (number of time domain samples)';...
      ...
      'array, [OutputVariance,InputVariance,CovVector]';...
      ...
      'vector, nonexcited even frequuencies for error determination';...
      'vector, nonexcited odd frequuencies for error determination';...
      'vector, error squares at nonexcited even output lines';...
      'vector, error squares at nonexcited odd output lines';...
      ...
      'array, Variance/2 for both arrays and cells';...
      'array or vector: old Fourier data';...
      'Hidden propery: iddat object'...
    };
end

if ~any(findstr(mode,'child'))
  [props2,values2]=idpropch(iddat,'',mode); %properties of iddat
  %Eliminate Version
  ind=find(props=='|');
  vi=findstr('|Version|',props);
  props(vi+[0:7])='';
  props=[props2,props(2:end)];
  if nargout>1
    id=find(ind==vi);
    values(id)=[];
    values=[values2;values];
  end
  %Replace SampleNumber
  ind=find(props=='|');
  vi=findstr('|SampleNumber|',props);
  if ~isempty(vi), props(vi+[1:2])=[]; props(vi+[1:10])='FreqNumber'; end
end

if any(findstr(mode,'set')) %eliminate properties from list
  elimprops={'OldTbSisoVariance','Fdat','AllFreqPoints','FreqIndices'};
  for ii=1:length(elimprops)
    propn=elimprops{ii};
    ind=find(props=='|');
    vi=findstr(['|',propn,'|'],props);
    if ~isempty(vi)
      props(vi+[0:length(propn)])='';
      if nargout>1, id=find(ind==vi); values(id)=[]; end  
    else
      error(['Property ''',propn,''' not found'])
    end
  end %for ii
end

ind=find(props=='|'); pno=length(ind)-1;
if (nargout>1)&(length(values)~=pno)
  error('Different number of properties and values')
end
%
if (nargin==1)|isempty(property), return, end

if ~isempty(property) %Property to be determined
  if strncmpi(property,'Variance',length(property))
    error(sprintf(['Property ''',property,''' is obsolete, and has been eliminated.\n',...
        '  Replace property ''',property,''' by ''SisoVariance''']));
    property=['Siso',property];
  elseif strncmpi(property,'OldTbVariance',length(property))
    error(['Property ''',property,''' was changed to ''OldTbSisoVariance''']);
    property='OldTbSisoVariance';
  end
  if strncmpi(property,'InputData',7)
    property='Input';
  elseif strncmpi(property,'OutputData',8)
    property='Output';
  elseif strcmpi(property,'InputD')
    indpstr=[2,3];
  elseif strcmpi(property,'OutputD')
    indpstr=[2,3];
  else
    indpstr=findstr(lower(props),['|',lower(property)]);
  end
  if strcmp(lower(property),'input')|strcmp(lower(property),'output')
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  end
  if strcmp(lower(property),'u')|strcmp(lower(property),'y') %u or y
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  elseif strcmp(lower(property),'n') %N
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  elseif strcmp(lower(property),'groups') %Groups
    indpstr=findstr(lower(props),['|',lower(property),'|']);
  elseif (length(property)>1)&any(findstr(lower(property(1)),'yu'))&~strncmpi(property,'use',3)
    if strcmp(lower(property(1)),'u'), property=['input',property(2:end)];
    elseif strcmp(lower(property(1)),'y'), property=['output',property(2:end)];
    end
    indpstr=findstr(lower(props),['|',lower(property)]);
  end
  if isempty(indpstr)
    if strcmp(mode,'set')
      error(['Unidentifiable fiddata property ''',property,''' for the set command'])
    else
      error(['Unidentifiable fiddata property ''',property,''''])
    end
  else %not empty
    if indpstr(1)==1, indpstr(2)=[]; end %eliminate Names ambiguity
    if length(indpstr)>1
      error(['Ambiguous fiddata property ''',property,''''])
    else
       indi=min(find(ind>indpstr)); inde=ind(indi);
       props=props(indpstr+1:inde-1);
       if nargout>1
          no=length(find(ind<=indpstr)); values=values{no};
          if strcmp(props,'Synchronization')&(get(dat,'expnumber')<=1)
            values='[ ''empty string for experiment number <= 1'' ]';
          end 
          indp=find(ind==indpstr);
       end
    end
  end
end
%
% end @fiddata/idpropch.m