function result = subsref(model,Struct)
%SUBSREF Referencing for objects.
%          model.fieldname
%          model.fieldname(indices) 
%          model.fieldname(i,j)
%
%       See also: GET.

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1998-99
%       All rights reserved.
%       $Revision: $
%       Last modified: I. Kollar, 13-Aug-1999

ni = nargin;
if ni==1,
   result = model;
   return
end

%StructL = length(Struct);
switch Struct(1).type
case '.'
   % At least model.fieldname
   % Output will be a (piece of) one of the system fields
   result = get(model,Struct(1).subs);
   if length(Struct)>1
     result=subsref(result,Struct(2:end));
   end
   return
case '()'
   % model(something)
   % Fall through to code below
case '{}'
   error('Cell contents reference from a non-cell array object')
otherwise
   error(['Unknown type: ' Struct(1).type])
end


% Everything else just handles fidmodel(something)
indices = Struct(1).subs;
if ~strcmp(indices{1},':')|((length(indices)>=2)&~strcmp(indices{2},':'))
  num=model(1).num; denom=model(1).denom;
  if ~iscell(num), num={num}; end, if ~iscell(denom), denom={denom}; end
  endi=length(indices); ind=indices(1:min(2,endi));
  try, num=num(ind{:});
  catch, index=ind, error('Cannot reference to numerator from index')
  end
  if length(denom)>1, denom=denom(ind{:}); end
  model(1).num=num{1}; model(1).denom=denom{1};
  result=model;
  %error('Subsref according to input or output channels is not implemented yet')
end
%Select I/O channels***

if length(indices)>=3
  models=struct(model);
  models=models(indices{3:end});
  result=fidmodel(models);
end
if length(Struct)>1,
  result=subsref(result,Struct(2:end));
end
% end ../@tf/subsref.m
