function idobj=iddata(obj)
%IDDATA  Transform fiddata or tiddata object to iddata object (ident toolbox)
%
%       idobj=IDDATA(fidobj);

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1997-2001
%       All rights reserved.
%       $Revision: $
%       Last modified: 08-Aug-2001

ni = nargin;
error(nargchk(1,1,ni))
no = nargout;
if no>1, error('Only one output argument is allowed'), end
if exist('@iddata/iddata.m') %iddata found
  icn=1; ocn=1; en=get(obj,'expn');
  input=get(obj,'input');
  %if ~iscell(input), input={input}; end
  if iscell(input), [icn,en]=size(input); end
  output=get(obj,'output');
  %if ~iscell(output), output={output}; end
  if iscell(output), [ocn,en]=size(output); end
  try
    idobj=iddata(output,input);
  catch
    %warning('iddata cannot accept cell array input')
    if iscell(input)
      [v,h]=size(input);
      if (v>1)&(h==1), input=cat(2,input{:}); end
    end
    if iscell(output)
      [v,h]=size(output);
      if (v>1)&(h==1), output=cat(2,output{:}); end
    end
    idobj=iddata(output,input);
  end
  if isa(obj,'fiddata')
    Domain='Frequency';
    fs=get(obj,'fs');
    if isempty(fs), Ts=[]; Fs=[]; else Ts=1/fs; Fs=fs; end
  elseif isa(obj,'tiddata')
    Domain='time';
    Ts=get(obj,'Ts'); Fs=[];
  end
  if isempty(Ts), Tsset=inf; else Tsset=Ts; end %temporary value
  if isempty(Fs), Fsset=inf; else Fsset=Fs; end %temporary value
  ver=version;
  if isa(obj,'fiddata')&(str2num(ver(1:3))>=6.5)
    if ~strcmp(ver(1:5),'6.5.0') %newer than or equal to 6.5.1
      idobj=iddata(output,input,Tsset,'FREQ',get(obj,'FreqPoints'));
    else
      idobj=iddata(output,input,Fsset,'Domain',Domain);
    end
  else
    idobj=iddata(output,input,Tsset,'Domain',Domain);
  end
  vid=get(idobj,'version');
  if strcmp(vid(1),'0')|all(vid(1:3)<='1.0')
    %Conversion OK
  else
    disp('iddata properties may have been changed.')
    error(['  Recent iddata version: ',vid,', known version: 1.0'])
  end
  %
  name=get(obj,'name');
  if ~isempty(name), set(idobj,'name',name), end
  enam=get(obj,'ExperimentName');
  if ~isempty(enam)
    if isstr(enam), enam={enam}; end
    set(idobj,'experimentname',enam)
  end
  %
  iu=get(obj,'inputunit');
  if ~isempty(iu), set(idobj,'inputunit',iu), end
  ou=get(obj,'outputunit');
  if ~isempty(ou), set(idobj,'outputunit',ou), end
  in=get(obj,'inputname'); %if isstr(in)&(length(in)>0), in={in}; end
  if ~isempty(in), set(idobj,'inputname',in), end
  on=get(obj,'outputname'); %if isstr(on)&(length(on)>0), on={on}; end
  if ~isempty(on), set(idobj,'outputname',on), end
  pl=get(obj,'periodlength');
  if ~isempty(pl)&~isnan(pl), set(idobj,'period',pl), end
  ic=get(obj,'inputcharacter');
  if ~isempty(ic)
    if strcmpi(ic,'BL')|strcmpi(ic,'AASamples'), ic='bl'; end
    if isstr(ic), ic={lower(ic)}; end
    if (icn>1)|(en>1)
      if isstr(ic), ic={ic}; end
      while size(ic,1)<icn, ic=[ic;ic(end,:)]; end
      while size(ic,2)<en, ic=[ic,ic(:,end)]; end
    end
    if iscell(ic)
      for ii=1:prod(size(ic))
        ic{ii}=lower(ic{ii});
        if strcmpi(ic{ii},'samples')|strcmpi(ic{ii},'discrete'), ic{ii}='bl'; 
        elseif strcmpi(ic{ii},'zohf'), ic{ii}='zoh';
        end
      end
    end
    set(idobj,'intersample',ic);
  end    
  notes=get(obj,'notes');
  if ~isempty(notes), set(idobj,'notes',notes), end
  ud=get(obj,'userdata');
  if ~isempty(ud), set(idobj,'userdata',ud), end
  if isa(obj,'fiddata')
    st=get(obj,'freqpoints');
    vm=version;
    if str2num(vm(1:3))>=6.5
      try, set(idobj,'frequency',st)
      catch, warning('Cannot set frequency')
      end
    else
      try, set(idobj,'samplingfrequencies',st)
      catch warning('Cannot set samplingfrequencies')
      end
    end
    try, set(idobj,'frequencyunit','Hz')
    catch warning('Cannot set frequencyunit')
    end
    %set(idobj,'fs',fs)
    InputDelay=get(obj,'InputDelay');
    OutputDelay=get(obj,'OutputDelay');
    if ~isempty(InputDelay), tst=InputDelay; else tst=0; end
    if ~isempty(OutputDelay), tst=tst-OutputDelay; end
  elseif isa(obj,'tiddata')
    set(idobj,'timeunit','s')
    tst=get(obj,'tstart');
    if isnumeric(tst)|(size(tst,1)==1), set(idobj,'TStart',tst);
    elseif isempty(tst) %nothing to do
    elseif iscell(tst)
      warning('Cannot properly handle different TStart values for different channels')
      [h,v]=size(tst);
      tst=cat(1,tst{:});
      tst=reshape(tst,h,v);
      if h>1, tst=mean(tst); end
      if size(tst,2)>1, tst=num2cell(tst); end
      set(idobj,'TStart',tst);
    end
    %
    st=get(obj,'samplinginstants');
    if ~isempty(st)
      set(idobj,'Ts',Ts,'samplinginstants',st)
    end
  end
else %iddata directory is missing, structure is generated
  warning('iddata.m not found: structure is generated')
  error('Not yet ready')
end

% end ../@iddat/iddata.m
