function [out1,out2,out3]=fdident(varargin)
%FDIDENT Start fdident GUI: alternative to fdtool
%       For details, see  help fdtool.

%       Special calls:
%       fdident('pcode',action)
%         action =
%           empty or 'generate': generate P-files beside M-files
%           'delete': remove toolbox-related P-files
%       fdident preload
%         Preload largest files to speed up later run
%       fdident('private','orthopol',...)
%         Call orthopol (not necessary for regular users)
%
%       Examples:
%           fdident
%           fdident('session','emachses.mat')
%           fdident pcode
%           fdident pcode delete
%           fdident preload
%
%       See also: FDTOOL.

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1997-2004
%       All rights reserved.
%       $Revision: $
%       Last modified: 21-Mar-2004

if nargin>=1
  if strcmpi(varargin{1},'pcode')
    %pcode calls
    fdpcode(varargin{2:end})
    return
  elseif strcmpi(varargin{1},'preload')
    %Speed up run by preloading largest files
    disp('Preloading large toolbox functions...')
    t0=clock;
    elis('preload')
    ploteltf('preload')
    varanal('preload')
    stdpz('preload')
    plotelpz('preload')
    iterctrl('preload')
    autoorder preload
    fprintf('  Elapsed time: %.1f s\n',etime(clock,t0))
    %
    disp('Preloading main object functions...')
    t0=clock;
    f=fiddata(1,2,3); f.version=f.version;
    t=tiddata(1,2,3); t.version=t.version;
    m=fidmodel('s',1,[2,3]); m.version=m.version; mm=stack(1,m,m);
    fprintf('  Elapsed time: %.1f s\n',etime(clock,t0))
    %
    %fdtool('preload')
    %
    return
  elseif strcmpi(varargin{1},'install')|strcmpi(varargin{1},'uninstall')
    if strncmp(version,'5.2',3)
      fdcfs5_2(varargin{:})
      if strcmpi(varargin{1},'uninstall')
        fdcfs5_2 clear
      end
    else
      fdcfs(varargin{:})
      if strcmpi(varargin{1},'uninstall')
        fdcfs clear
        %clear fdcfs
      end
    end
    return
  elseif strncmp(varargin{1},'license',3)
    if strncmp(version,'5.2',3)
      if nargout>=1, out1=fdcfs5_2(varargin{:});
      else fdcfs5_2(varargin{:})
      end
    else
      if nargout>=1, out1=fdcfs(varargin{:});
      else fdcfs(varargin{:})
      end
    end
    return
  elseif strcmpi(varargin{1},'addpath_fdident')
    addpath_fdident;
    return
  end
end
%
% ###############  P R I V A T E   F U N C T I O N   W R A P P E R  #################
%  call form: [out1,out2]=fdident('private',funcname,parameters)
if (nargin>=2)&strcmpi(varargin{1},'private')
  varargin=varargin(2:end);
  if isnumeric(varargin{1}), varargin{1}=setstr(varargin{1}); end
  if nargout==0, feval(varargin{:});
  elseif nargout==1, out1=feval(varargin{:});
  elseif nargout==2, [out1,out2]=feval(varargin{:});
  elseif nargout==3, [out1,out2,out3]=feval(varargin{:});
  else error('Too many output arguments')
  end
  return
end
% ###################################################################
%Pass arguments to fdtool
if length(dbstack)<4
  if nargout==0
    fdtool(varargin{:})
  elseif nargout==1
    out1=fdtool(varargin{:});
  else
    error('Too many output arguments required')
  end
else
  error('Wrong call to fdtool or fdident')
end
%
%End of fdident

function fdpcode(action)
%FDPCODE  Generate/delete P-files for fdident toolbox
%
%       Input arguments:
%       action =
%         empty or 'generate': generate P-files beside M-files
%         'delete': remove toolbox-related P-files
%         'here': generate P-files in current directory
%         <directory>: generate P-files in directory

clear functions
if nargin<1, action=''; end
if isempty(action), action='generate';
elseif ~any(strmatch(action,{'generate','delete'}))
  %Maybe directory?
  if isdir(action)
    disp(['Directory: ''',action,''''])
    error('This call is not ready yet')
  else
    error(['Parameter not allowed: ''',action,''''])
  end
end
%
files={'elis','fdwindef',['@iddat',filesep,'help'],...
    ['@fiddata',filesep,'help'],['@tiddata',filesep,'help'],...
    ['@fidmodel',filesep,'help']};
dirs={}; ext='';
if strcmp(action,'generate')
  disp('Generate P-files for fdident functions')
  ext='m';
elseif strcmp(action,'delete')
  disp('Delete P-files for fdident functions')
  ext='p';
end
%
for filecell=files(:)'
  file=filecell{1};
  w=which([file,'.',ext]);
  if isempty(w)&strcmp(ext,'p')
    disp(['No P-file found'])
    w=which([file,'.','m']);
  end
  if ~isempty(w)
    ind=find(w==filesep);
    dirpath=w(1:ind(end)-1);
    done=0;
    for ii=1:length(dirs)
      if strcmp(dirs{ii},dirpath), done=1; end
    end %for ii
  else
    done=1;
  end
  if done==0
    dirs=[dirs;{dirpath}];
    disp(['Directory ''',dirpath,''''])
    %
    if strcmp(action,'generate')
      cb='@';
      if strncmp(version,'5.',2)
        if isvms, cb='#';
        end
      end
      ind=find(file==cb);
      if ~isempty(ind)
        ind2=find(file==filesep);
        nameh=[file(ind+1:ind2-1),'h'];
      else
        nameh='';
      end
      struc=dir([dirpath,filesep,'*.',ext]);
      for ii=1:length(struc)
        name=struc(ii).name;
        if strcmpi(name,[nameh,'.',ext])|strcmpi(name,'fdident.m')...
            |strcmpi(name,'siglab.m')|strcmpi(name,'fdhwsigl.m')...
            |strcmpi(name,'contents.m')|strcmpi(name,'readme.m')
          %file <class>h.m is not a true M-file
          %preparsing of fdident is not possible when running fdident
          %preparsing of siglab may clash M-file under development
          %preparsing of fdhwsigl may clash M-file under development
          disp(['  Skipping file ''',name,''''])
        elseif strncmp(version,'5.2',3)&...
            ( any(findstr(file,'iddat'))&...
            (strcmp(lower(name),'set.m')|...
            strcmp(lower(name),'horzcat.m')|...
            strcmp(lower(name),'vertcat.m')|...
            strcmp(lower(name),'diff.m')|...
            strcmp(lower(name),'plus.m')|...
            strcmp(lower(name),'subsref.m')|...
            strcmp(lower(name),'subsasgn.m')) |...
            ( strcmp(lower(name),'ploteltf.m')|...
            strcmp(lower(name),'diff.m') ) ...
          )
          %strange error in version 5.2: some subfunctions
          %do not translate properly
          disp(['  Skipping file ''',name,''''])
        else
          disp(['  Parsing file ''',name,''''])
          if ii==1
            %make first try
            eval('pcode([dirpath,filesep,name],''-inplace''), pok=1;',...
              'pok=0;')
            if pok==0
              disp(['Cannot parse file ''',name,''''])
              break
            end
          else
            pcode([dirpath,filesep,name],'-inplace')
          end
        end
      end %for ii
    elseif strcmp(action,'delete')
      delete([dirpath,filesep,'*.p'])
      dp=dir([dirpath,filesep,'*.p']);
      if ~isempty(dp)
        error(['Could not delete files ''',dirpath,filesep,'*.p'''])
      end
      clear functions %make sure P-files disappear from memory
    end
  end
end %for filecell
%
if strcmp(action,'delete')
  %Last, check uniqueness of files
  [x,msg]=fdunique(1);
  if x==0, disp(msg), error, end
elseif strcmp(action,'generate')
  if strncmp(version,'5.2',3)
    disp(str2mat('Warning: Some subfunctions (second functions in function M-files) compile',...
      'incorrectly under Matlab 5.2. We made sure in this command that such',...
      'files do not get pcode-ed. Should you see however an error message that a',...
      'function or a variable is not found in a P-file, simply delete this P-file ',...
      '( delete(which(''filename'')) ), and this will fix the problem.',...
      'If the problems repeat, simply delete all P-files (''fdident pcode delete'').'))
  end
end
%
%End of fdpcode

%
%End of file
