function fddgui1(mode)
%FDDGUI1  Open window to run all fdident GUI demonstrations.
%INPUT CHANNEL TEMPORARILY ELIMINATED!
%
%       If mode='intro_gui', a simple window comes up with GUI demos,
%       If mode='intro_ident', a simple window comes up with ID demos
%       If mode='test', a palette of tests comes up,
%       else a complex one.

%       Copyright (c) I. Kollar and Vrije Universiteit Brussel, ELEC, 1997-2001
%       All rights reserved.
%       $Revision: $
%       Last modified: 07-Jun-2001, I. Kollar

if nargin<1, mode=''; end
%demos related to measurement and simulation
meas_demos={...
    'fdhwemul.mat'...
    ,'notchmeas.mat'...
  };
sim_demos={...
    'ruidsim.mat'...
    ,'tsffdsim.mat'...
    ,'tstfdsim.mat'...
    ,'tstcorrt.mat'...
  };
elim_demos={};
if ~guiinfos('ismeasurement'), elim_demos=[elim_demos,meas_demos]; end
if ~strcmpi(guiinfos('userlevel'),'Advanced')
  elim_demos=[elim_demos,sim_demos];
end
%
if strcmp(mode,'intro_gui')
  labelList = str2mat( ...
    'Helps'...
    ,'Process time domain data'...
    ,'Process freq. domain data'...
    ,'Generate default session'...
    ,'Compose data objects (GUI)'...
    ,'Use of the compose window'...
    ,'Create data objects (commands)'...
    ,'Nonlinearity making multi-experiment'...
    ,'Nonlinearity using special multisine'...
    );
  
  nameListown={ ...
      'starting.mat'...
      ,'introtdd.mat'...
      ,'introfdd.mat'...
      ,'fddefpre.mat'...
      ,'compdemo_intro.mat'...
      ,'compdemo_timefreq.mat'...
      ,'objects.mat'...
      ,'nonlindemo1.mat'...
      ,'nonlindemo2.mat'...
    };
    message=...                                                       
    ['These examples illustrate the use of the Frequency ',...
      'Domain System Identification Toolbox, ',...
      'equipped with a graphical user interface, working with measured Data. ',...
      'All examples are based on an action recorder which operates similarly ',...
      'to a tape recorder. When it stops, any of the controls of the ',...
      'GUI can be changed. For proper continuation of a demonstration, the ',...
      'recorder should be started from the state as it stopped in.'];
  
elseif strcmp(mode,'intro_ident')
  labelList = str2mat( ...
    'Automatic identification'...
    ,'Determine a resistance'...
    ,'Determine R from simulated data'...
    ,'Emulate data, identify system'...
    ,'Identify a notch filter'...  
    ,'Measure & identify notch filter'...  
    ,'Detailed identification example'...
    ,'Automatic order selection'...
    ,'Nonlinearity making multi-experiment'...
    ,'Nonlinearity using special multisine'...
);
  
  nameListown={ ...
      'autodemo.mat'...
      ,'ruidemo.mat'...
      ,'ruidsim.mat'...
      ,'fdhwemul.mat'...
      ,'notchdem.mat'...
      ,'notchmeas.mat'...
      ,'lowpdem2.mat'...
      ,'autoorder_demo.m'...
      ,'nonlindemo1.mat'...
      ,'nonlindemo2.mat'...
    };
    message=...                                                       
    ['These examples illustrate how to do identification using '... 
    'the Frequency Domain System Identification Toolbox ',...
      'with a graphical user interface. ',...
      sprintf('\n'),...
      'Significantly new examples are written in red.'];
  
elseif strcmp(mode,'test')
  labelList = str2mat( ...
    'Test Automatic UserLevel'...
    ,'Test of arrow actions'...
    ,'Excit. signal test'...
    ,'Test of ZOH precompensation'...
    ,'Test of object composition'...
    ,'Test of time domain simulation'...
    ,'Test of freq. domain simulation'...
    ,'Test of ECPM'...
    ,'Test of ECPM, complex coeffs'...
    ,'Test of correlation test'...
    ,'Test of variances with OP'...
    ,'Test of transients with OP'...
    ,'Test of measurement (VNA)'...
    ,'Test of nonlinear simulation I.'...
    ,'Test of nonlinear simulation II.'...
    );
  
  nameListown={ ...
      'testautolevel.mat'...
      ,'testarr.mat'...
      ,'testessd.mat'...
      ,'tstessda.mat'...
      ,'testcompose.mat'...
      ,'tstfdsim.mat'...
      ,'tsffdsim.mat'...
      ,'testecpm.mat'...
      ,'testecpc.mat'...
      ,'tstcorrt.mat'...
      ,'testoppz.mat'...
      ,'testtrop.mat'...
      ,'testvna.mat'...
      ,'testnl1levels.mat'...
      ,'testnl2levels.mat'...
    };
    message=...                                                       
    ['These are the currently available tests for fdident GUI windows.'];

else %complex
  labelList = str2mat( ...
    'Bandpass filter'...
    ,'Electrical machine'...
    ,'Input channel'...
    ,'Lowpass filter, z-domain'...
    ,'Airplane cabin'...  
    ,'Mech. system: aluminum plates'...
    ,'Mech. system: glassfiber plates'...
    ,'Mech. system: crankcase'...
    ,'Flexible robot arm'...
    ,'Wilkinson-type system'...
    ,'Deal with transient data'...
    ,'Corrosion modeling: use of w=sqrt(s)'...
    ,'Scan delays'...
    ,'Nuclear Magnetic Resonance'...
    );
  
  nameListown={ ...
      'bandpdem.mat'...
      ;'emachdem.mat'...
      ;'inpchdem.mat'...
      ;'lowpdem.mat'...
      ;'cabindem.mat'...
      ;'alupldem.mat'...
      ;'glfibdem.mat'...
      ;'crankdem.mat'...
      ;'rarmdemo.mat'...
      ;'wilkdemo.mat'...
      ;'bptrdemo.mat'...
      ;'corrosdm.mat'...
      ;'delaydem.mat'...
      ;'nmrdemo.mat'...
    };
  
  %figureFlagList = [ones(1,size(nameList,2))];
  message=...                                                       
    ['These examples illustrate the power and the use of the Frequency ',...
      'Domain System Identification Toolbox, ',...
      'equipped with a graphical user interface, ',...
      'performing identification on real-life data. ',...
      'All examples are based on an action recorder which operates similarly ',...
      'to a tape recorder. When it stops, all the controls of the ',...
      'GUI can be used. For proper continuation of a demonstration, the ',...
      'recorder should be started from a similar state as it stopped in.'];
end

if ~isempty(elim_demos)
  %eliminate some demos that may not be called
  for iii=1:length(elim_demos)
    for iv=length(nameListown):-1:1
      if strcmpi(elim_demos{iii},nameListown{iv})
        nameListown(iv)=[];
        labelList(iv,:)=[];
      end
    end
  end
end
if ~exist('siglab.dll')|~isequal(fdhwcall('hw_chk','SigLab'),1)
  %SigLab does not exist or it is off
  for sln={'notchmeas.mat','testvna.mat'};
    iv=strmatch(sln,nameListown);
    if ~isempty(iv)
      nameListown(iv)=[];
      labelList(iv,:)=[];
    end
  end
end
%
if isempty(labelList)
  message='Sorry! These demos are currently under preparation';
end
nameList='';
for ii=1:length(nameListown)
  dfname=nameListown{ii};
  if exist(dfname)
    %disp(['Taking file ''',dfname,''''])
    if strcmpi('.m',dfname(end+[-1:0])) %M-file
      nameList=str2mat(nameList,dfname(1:end-2));
    else
      if strcmp(mode,'test'), rmode='development';
      else
        %rmode='demo';
        rmode='recorder';
      end
      nameList=str2mat(nameList,...
        ['if isempty([findall(0,''tag'',''fdtool_recorder_fig'');',...
          'findall(0, ''tag'',''guirecrd_fdtool'')]),',...
          'fdtool(''callback'',''guirecrd'',''init'',''',rmode,'''), ',...
          'else figure([findall(0,''tag'',''fdtool_recorder_fig'');',...
              'findall(0, ''tag'',''guirecrd_fdtool'')]),end, ',...
          'fdtool(''callback'',''guirecrd'',''SetContinuous'',1),',...
          'fdtool(''callback'',''guirecrd'',''SetDiscardPause'',0),',...
          'fdtool(''callback'',''guirecrd'',''loadhist'',''',dfname,''')']); 
    end
  else
    warning(['History file ''',dfname,''' does not exist'])
  end
end %for ii
if size(nameList,1)>0, nameList(1,:)=''; end

fdguidw(labelList,nameList,message); %Open window for demos
%
%End of fddgui1
